﻿(function () {
    'use strict';

    function detailController($scope, $routeParams, $window,
        localizationService,
        translateProviderService,
        translateNodeService,
        translatorService,
        notificationsService)
    {
        var vm = this;
        vm.loaded = false;
        vm.page = {
            title: "detail",
            description: "detail description"
        };

        vm.node = {};

        vm.editValue = editValue;
        vm.saveValue = saveValue;
        vm.cancel = cancelEdit;

        vm.copySource = copySource;

        vm.isValue = isValue;

        vm.saveAll = save;
        vm.cancelAll = cancel;

        getSettings($routeParams.id);
        // loadNode($routeParams.id);

        //////////////////

        function getSettings(nodeId) {
            translateProviderService.getSettings("C2E08D64-7BAC-4294-B071-924B4EA3A77B")
                .then(function (result) {
                    vm.settings = result.data;
                    initEditorConfig(vm.settings);
                    loadNode(nodeId);
                });
        }

        function initEditorConfig(settings) {

            vm.editorConfig = {
                toolbar: ["code", "undo", "redo", "cut", "bold", "italic"],
                stylesheets: [],
                dimensions: { height: 300, width: '100%' },
                plugins: "noneditable"
            };

            if (settings.custom !== undefined && settings.custom !== '') {
                vm.editorConfig = JSON.parse(settings.custom);
                return;
            }

            if (settings.toolbar !== undefined && settings.toolbar !== '') {
                vm.editorConfig.toolbar = JSON.parse("[" + settings.toolbar + "]");
            }

            if (settings.stylesheets !== undefined && settings.stylesheets !== '') {
                vm.editorConfig.stylesheets = JSON.parse("[" + settings.stylesheets + "]");
            }

            if (settings.height !== undefined && settings.height !== '') {
                vm.editorConfig.dimensions.height = parseInt(settings.height);
            }

            return;
        }



        function loadNode(nodeId) 
        {
            translatorService.getNode(nodeId)
                .then(function (result) {
                    vm.loaded = true;
                    vm.node = result.data;

                    vm.page.title = vm.node.MasterNodeName
                        + " to "
                        + vm.node.TargetNodeName;

                    localizationService.localize("translate_translation")
                        .then(function (value) {
                            vm.page.description = vm.node.Culture.DisplayName + " " + value;
                        });

                    vm.node.editable = false; 
                    if (vm.node.Status == 'InProgress' || vm.node.Status == 'Updated') {
                        vm.node.editable = true;
                    }

                    makeValueProperties(vm.node.Groups);

                });
        }       

        function saveValue(value) {
            if (value.HtmlControl) {
                value.Value = value.editor.value;
            }
            else {
                value.Value = value.editValue;
            }

            value.Translated = true;
            value.editing = false; 
        }

        function editValue(value) {
            value.editing = true;
            if (value.HtmlControl) {
                value.editor.value = value.Value;
                value.editor.onValueChanged(value.editor.value, '');
            }
            else {
                value.editValue = value.Value;
            }
        }

        function copySource(source, target) {
            target.Value = source.Value;
            makeValue(target);
            target.Translated = false;
        }

        function cancelEdit(value) {

            if (value.HtmlControl) {
                value.editor.value = value.Value;
            }

            value.Translated = false;
            value.editing = false;
        }

        function save() {
            // do the full save back to the system here...
            translatorService.updateProperties(vm.node.Id, vm.node.Groups)
                .then(function (result) {
                    notificationsService
                        .success("Saved",
                            localizationService.localize("translateUpdates_properties")
                        );

                    // redirect back in history....
                    $window.history.back();

                }, function (error) {
                    notificationsService.error("Error", error.data.ExceptionMessage);
                });
        }

        function cancel() {
            $window.history.back();
        }

        function makeValueProperties(propertyGroups) {

            angular.forEach(propertyGroups, function (group, key) {
                angular.forEach(group.Properties, function (property, key) {
                    makeValue(property.Target);
                });
            });

        }

        function isValue(value) {
            if (value.InnerValues == undefined || angular.equals({}, value.InnerValues))
                return true; 

            return false;
        }

        function makeValue(value) {

            if (value.InnerValues && !angular.equals(value.InnerValues, {})) {
                angular.forEach(value.InnerValues, function (innerValue, key) {
                    makeValue(innerValue);
                });
            }
            else {
                if (value.HtmlControl) {
                    value.editValue = value.Value;
                    if (value.editor === undefined) {
                        value.editor = {
                            label: 'bodyText',
                            description: 'Load some stuff here',
                            view: 'rte',
                            config: {
                                editor: vm.editorConfig
                            },
                            value: value.editValue
                        };
                    }
                    else {
                        value.editor.value = value.editValue;
                        value.editor.onValueChanged(value.editor.value, '');
                    }
                }
            }
        };
    }

    angular.module('umbraco')
        .controller('translate.detailController', detailController);
})();